using Caliburn.Micro;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using VisualComponents.Create3D;
using VisualComponents.eCatalogue.DataModel;
using VisualComponents.eCatalogue.DataModel.Data;
using VisualComponents.eCatalogue.DataModel.Data.Entities;
using VisualComponents.UX.Shared;

namespace Plugin.ReadDWGBlock
{
    [Export(typeof(IActionItem))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class DWGBlockAttributeReader:ActionItem
    {
        private IApplication _app = null;
        private List<Item> eCatComponentCache = null;

        [ImportingConstructor]
        public DWGBlockAttributeReader(IApplication app) :
            base(@"DWGBlockAttributeReader", @"DWG Attributes Reader", "rGroup", null, MenuTools.BigButtonTool, true)
        {
            _app = app;
            this.canExecute = (() => true);
            this.ToolTipHeader = "DWG Block Attributes Reader";
            this.ToolTipContent = "Select a component that has DWG Attributes";
            ContextFilter = Contexts.Configure;
        }

        public class DWGAttribute
        {
            public string Tag { get; set; }
            public string Value { get; set; }
            public string Position { get; set; }
        }

        public class DWGBlock
        {
            public string BlockName { get; set; }
            public int Id { get; set; }
            public string Position { get; set; }
            public List<DWGAttribute> Attributes { get; set; } = new List<DWGAttribute>();
        }

        public override void Execute()
        {
            _previousComponent = null;
            previous_comp_interfece = null;
            var comp = _app.SelectionManager.GetSelected<ISimComponent>();
            if (comp != null)
            {
                if (eCatComponentCache == null)
                {
                    eCatComponentCache = EnumerateEcatComponents();
                }

                // DWGAttributes component property contains the JSON serialized data of the block attributes
                // found in the dwg file used to create this component
                var attrProp = comp.GetProperty("DWGAttributes") as IStringProperty;
                if (attrProp != null)
                {
                    _app.SelectionManager.Clear();
                    var blocks = JsonSerializer.Deserialize<List<DWGBlock>>(attrProp.Value);
                    foreach (var block in blocks)
                    {
                        Matrix mat = Matrix.Parse(block.Position);
                        // get type: determines the component from the eCat
                        var type = block.Attributes.FirstOrDefault(item => item.Tag == "TYPE");
                        var name = block.Attributes.FirstOrDefault(item => item.Tag == "NAME");
                        if (type != null && name != null)
                        {
                            var eCatItem = eCatComponentCache.FirstOrDefault(item => item.Type.ToLower() == type.Value.ToLower() && item.Name.ToLower() == name.Value.ToLower());
                            if (eCatItem != null)
                                {
                                    var comps = _app.LoadLayout(new Uri(eCatItem.FileUri));
                                    if (comps.Length > 0)
                                    {
                                        var component = comps[0];
                                        var componentProperties = component.Properties;
                                        component.TransformationInWorld = mat;
                                    //get properties: for other attribute we try to find matching component property
                                    var attributes = block.Attributes.Where(item => item.Tag != "TYPE" && item.Tag != "NAME");
                                    if (attributes != null)
                                    {
                                       foreach (var attr in attributes)
                                        {
                                            var prop = componentProperties.FirstOrDefault(item => item.Name.ToLower() == attr.Tag.ToLower());
                                           if (prop != null)
                                           {
                                                prop.Value = double.Parse(attr.Value);
                                            }
                                      }
                                    }

                                }
                                }

                        }
                    }
                }
            }
        }

        private List<Item> EnumerateEcatComponents()
        {
            var eCatContext = IoC.Get<IECatDataContext>();
            var filter = eCatContext.EnabledItems;
            //Get all the components to a list
            return filter.ToList().Where(i => !i.IsDeprecated && i.ModelType == "Component").ToList();
        }
    }
    
    [Export(typeof(IPlugin))]
    internal class DWGBlockAttributeReaderPlugin : IPlugin
    {
        public void Exit()
        {
            
        }

        public void Initialize()
        {
            ICommandRegistry command = IoC.Get<ICommandRegistry>();
            var actionitem = command.FindItem("DWGBlockAttributeReader");
            var setup = new UXSiteSetup
            {
                EntryId = actionitem.Id,
                EntryCaption = actionitem.Content,
                UXSiteIdPath = "VCTabHome/DWGReader"
            };
            command.RegisterActionItem(actionitem, setup);
        }
    }
}
